@@ -54,7 +54,8 @@ jags.model <- function(file, data=sys.frame(sys.parent()), inits,
 
     p <- .Call("make_console", PACKAGE="rjags")
     .Call("check_model", p, modfile, PACKAGE="rjags")
-
+    unlink(modfile)
+    
     varnames <- .Call("get_variable_names", p, PACKAGE="rjags")
     if (is.environment(data)) {
         ##Get a list of numeric objects from the supplied environment
@@ -65,7 +66,7 @@ jags.model <- function(file, data=sys.frame(sys.parent()), inits,
     }
     else if (is.list(data)) {
         v <- names(data)
-        if (is.null(v)) {
+        if (is.null(v) && length(v) != 0) {
             stop("data must be a named list")
         }
         if (any(nchar(v)==0)) {
@@ -81,6 +82,21 @@ jags.model <- function(file, data=sys.frame(sys.parent()), inits,
         for (i in seq(along=unused.variables)) {
             warning("Unused variable \"", unused.variables[i], "\" in data")
         }
+        ### Check for data frames
+        df <- which(as.logical(sapply(data, is.data.frame)))
+        for (i in seq(along=df)) {
+            if (all(sapply(data[[df[i]]], is.numeric))) {
+                #Turn numeric data frames into matrices
+                data[[df[i]]] <- as.matrix(data[[df[i]]])
+            }
+            else {
+                stop("Data frame with non-numeric elements provided as data: ",
+                     names(data)[df[i]])
+            }
+        }
+    }
+    else if (is.null(data)) {
+        data <- list()
     }
     else {
         stop("data must be a list or environment")
@@ -172,6 +188,12 @@ jags.model <- function(file, data=sys.frame(sys.parent()), inits,
                 stop("Invalid parameters for chain ", i)
             }
             setParameters(init.values[[i]], i)
+            unused.inits <- setdiff(names(init.values[[i]]), varnames)
+            unused.inits <- setdiff(unused.inits,
+                                    c(".RNG.seed", ".RNG.state", ".RNG.name"))
+            for (j in seq(along=unused.inits)) {
+                warning("Unused initial value for \"", unused.inits[j], "\" in chain ", i)
+            }
         }
     }
 
@@ -339,16 +361,19 @@ jags.samples <-
       stop("type must be a character vector")
 
     pn <- parse.varnames(variable.names)
-    .Call("set_monitors", model$ptr(), pn$names, pn$lower, pn$upper,
-          as.integer(thin), type, PACKAGE="rjags")
+    status <- .Call("set_monitors", model$ptr(), pn$names, pn$lower, pn$upper,
+                    as.integer(thin), type, PACKAGE="rjags")
+    if (!any(status)) stop("No valid monitors set")
     update(model, n.iter, ...)
     ans <- .Call("get_monitored_values", model$ptr(), type, PACKAGE="rjags")
     for (i in seq(along=ans)) {
         class(ans[[i]]) <- "mcarray"
     }
     for (i in seq(along=variable.names)) {
-      .Call("clear_monitor", model$ptr(), pn$names[i], pn$lower[[i]],
-            pn$upper[[i]], type, PACKAGE="rjags")
+        if (status[i]) {
+            .Call("clear_monitor", model$ptr(), pn$names[i], pn$lower[[i]],
+                  pn$upper[[i]], type, PACKAGE="rjags")
+        }
     }
     return(ans)
 }
@@ -535,3 +560,8 @@ isDLLLoaded <- function(file)
     }
     return(FALSE)
 }
+
+parallel.seeds <- function(factory, nchain)
+{
+    .Call("parallel_seeds", factory, nchain, PACKAGE="rjags")
+}
